/**
 * Copyright (c) 2010-2011, Chair for Communication and Distributed Systems,
 * RWTH Aachen University, Germany
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
 * REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
 * INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
 * LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
 * OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef HARDWARE_H
#define HARDWARE_H

/**
 *******************************************************************************
 * Control some TinyWifi behavior
 *******************************************************************************
 */

/**
 * LEDs
 */
#define __LEDS_QUIET //use single line display of LEDs
//#define __LEDS_SHUT_UP //silence any LED output to console

/**
 * Active Messaging
 */
//#define __LINUXNET_VERBOSE //be verbose when using radio ActiveMessaging

#ifndef TINYWIFI_PROTOCOL
#define TINYWIFI_PROTOCOL 0x5555
#endif

//#ifndef TINYWIFI_INTERFACE_NAME
#define TINYWIFI_INTERFACE_NAME "ath0"
//#endif

//#define __LINUXNET_UDPIP //use UDP/IP sockets (fallback only!)

/* <valid for __LINUXNET_UDPIP only> */
//sets the broadcast address used
#ifndef IP_BROADCAST_ADDRESS
//#define IP_BROADCAST_ADDRESS INADDR_BROADCAST
#define IP_BROADCAST_ADDRESS 0xA9FE09FF //169.254.9.255 (umic adhoc mesh)
#endif
//time to live (TTL) header value for UDP/IP packets
#ifndef TINYWIFI_TTL
#define TINYWIFI_TTL 1
#endif
//port for incoming UDP messages
#ifndef TINYWIFI_PORT
#define TINYWIFI_PORT 5555
#endif
/* </valid for __LINUXNET_UDPIP only> */

/**
 * Serial Active Messaging
 */
//#define __LINUXSERIAL_VERBOSE //be verbose when using serial ActiveMessaging
#define BAUD_NUM   115200 //numeric representation of the baudrate to be set
#define BAUD_TERM B115200 //BAUD_NUM preceded with a 'B'
                          //(used with Linux system calls)


/**
 *******************************************************************************
 * Signaling, atomic sections and interrupts
 *******************************************************************************
 */
#include <signal.h>

#define SIGRT01      SIGRTMIN+1 //wake up main thread
#define SIGRT02      SIGRTMIN+2 //wake up senderThread in LinuxActiveMessageC.nc
#define SIGRT03      SIGRTMIN+3 //wake up senderThread in PlatformSerialP.nc
#define SIGRT04      SIGRTMIN+4 //wake up UartByte.receive in PlatformSerialP.nc
#define SIGRT05      SIGRTMIN+5 //not yet used
#define SIGRT06      SIGRTMIN+6 //not yet used
#define SIGRT07      SIGRTMIN+7 //not yet used

uint8_t STATUS = 0x0;

typedef enum statusRegisterBit {
  GIE = 0, //Global Interrupts Enable
  GIP_SIGALRM = 1, //Global Interrupt (SIGALRM) Pending (used for LinuxTiming)
} status_bit_t;

typedef uint8_t __nesc_atomic_t;
typedef uint8_t mcu_power_t;

inline bool interruptsEnabled() {
  uint8_t *status = &STATUS;
  return (*status & (1 << GIE)) != 0;
}

inline bool isInterruptPending(status_bit_t which) {
  uint8_t *status = &STATUS;
  return (*status & (1 << which)) != 0;
}

inline void setStatusFlag(status_bit_t which) {
  uint8_t *status = &STATUS;
  *status |= (1 << which);
}

inline void clrStatusFlag(status_bit_t which) {
  uint8_t *status = &STATUS;
  *status &= ~(1 << which);
}

inline void __nesc_enable_interrupt() {
  uint8_t *status = &STATUS;
  *status |= (1 << GIE);
}

inline void __nesc_disable_interrupt() {
  uint8_t *status = &STATUS;
  *status &= ~(1 << GIE);
}

inline __nesc_atomic_t __nesc_atomic_start(void) @spontaneous() {
  __nesc_atomic_t result = interruptsEnabled();
  __nesc_disable_interrupt();
  return result;
}

inline void __nesc_atomic_end(__nesc_atomic_t reenableInterrrupts ) @spontaneous() {
  if ( reenableInterrrupts ) {
    __nesc_enable_interrupt();
    if ( isInterruptPending(GIP_SIGALRM) == TRUE ) {
      clrStatusFlag(GIP_SIGALRM);
      raise(SIGALRM);
    }
  }
}

inline void __nesc_atomic_sleep() { }

/* Floating-point network-type support */
typedef float nx_float __attribute__((nx_base_be(afloat)));

inline float __nesc_ntoh_afloat(const void *COUNT(sizeof(float)) source) @safe() {
  float f;
  memcpy(&f, source, sizeof(float));
  return f;
}

inline float __nesc_hton_afloat(void *COUNT(sizeof(float)) target, float value) @safe() {
  memcpy(target, &value, sizeof(float));
  return value;
}

// enum so components can override power saving, as per TEP 112.
enum {
  TOS_SLEEP_NONE = 0,
};

#endif
